# coding: utf-8

import json
from logging import config, getLogger
import urlparse
import requests

from webob.exc import HTTPForbidden

from tionix_scheduler_client import settings

config.dictConfig(settings.LOGGING)
logger = getLogger(__name__)


class SchedulerClientHttpException(Exception):
    """
    Raise this Exception when scheduler rest api response with errors
    """
    def __init__(self, msg, status_code):
        super(SchedulerClientHttpException, self).__init__(msg)

        self.msg = msg
        self.status_code = status_code

    def __str__(self):
        return self.msg


class Client(object):
    """
    Class-manager of scheduler's rest api.
    """
    BASENAME = 'timetable/'
    BASE_URL = urlparse.urljoin(settings.SCHEDULER_URL, BASENAME)
    MODULE_BASENAME = 'tnx-scheduler/'
    MODULE_BASE_URL = urlparse.urljoin(settings.SCHEDULER_URL, MODULE_BASENAME)

    def __init__(self, headers=None):
        self.headers = headers or {}
        self.headers['content-type'] = 'application/json'

    def add_header(self, key, value):
        self.headers[key] = value

    def _get(self, prefix=None, params=None):
        """
        Perform a host "action".
        """
        if prefix and not prefix.endswith('/'):
            prefix += '/'
        url = urlparse.urljoin(self.BASE_URL, prefix or '')
        r = requests.get(url, params=params or {}, headers=self.headers)
        return json.loads(r.content), r.status_code

    def _delete(self, prefix=None, params=None):
        """
        Perform delete request.
        """
        if prefix and not prefix.endswith('/'):
            prefix += '/'
        url = urlparse.urljoin(self.BASE_URL, prefix or '')
        r = requests.delete(url, data=json.dumps(params), headers=self.headers)
        return json.loads(r.content), r.status_code

    def get_tasks_list(self, object_type=None, groups=None, locale=None):
        """
        List of available tasks to schedule.
        """
        tasks_list = 'tasks_list/'
        logger.debug(u'Sending GET /{0}{1} object_type:{2} groups:{3} '
                     u'locale:{4}'.format(self.BASENAME, tasks_list,
                                          object_type, groups, locale))
        res, _status_code = self._get(tasks_list, {
            'type': object_type,
            'groups': groups,
            'locale': locale
        })
        logger.info(u'GET /{0}{1} success'.format(self.BASENAME, tasks_list))
        return res

    def get_monitoring_tasks_list(self, params):
        """
        List of celery running tasks from pending state.
        """
        tasks_monitoring = 'tasks_monitoring/'
        logger.debug(u'Sending GET /{0}{1} params:{2}'.format(
            self.BASENAME, tasks_monitoring, params))
        res, _status_code = self._get(tasks_monitoring, params)
        logger.info(u'GET /{0}{1} success'.format(self.BASENAME,
                                                  tasks_monitoring))
        return res

    def delete_monitoring_tasks_list(self, params):
        """
        Delete found tasks by params
        """
        tasks_monitoring = 'tasks_monitoring/'
        logger.debug(u'Sending DELETE /{0}{1} params:{2}'.format(
            self.BASENAME, tasks_monitoring, params))
        content, code = self._delete(tasks_monitoring, params)
        logger.info(u'DELETE /{0}{1} success'.format(self.BASENAME,
                                                     tasks_monitoring))
        return content, code

    def get_object_types_list(self):
        """
        List of object types available for schedule.
        """
        request = 'task_objects/'
        logger.debug(u'Sending GET /{0}{1}'.format(
            self.BASENAME, request))
        res, _status_code = self._get(request)
        logger.info(u'GET /{0}{1} success'.format(self.BASENAME,
                                                  request))
        return res

    def create_timetable(self, data):
        """
        Creates timetable with periodic task.
        """
        logger.debug(u'Sending POST /{0} data:{1}'.format(self.BASENAME, data))
        response = requests.post(self.BASE_URL, json=data,
                                 headers=self.headers)
        logger.info(u'POST /{0} successed with {1}:{2}'.format(
            self.BASENAME, response.status_code,
            response.content.decode('utf8')))
        return response

    def get_timetable(self, timetable_id):
        """
        Gets one timetable by ID.
        """
        res, status_code = self._get(str(timetable_id))
        logger.info(u'GET /{0}{1}/ success'.format(self.BASENAME,
                                                   timetable_id))

        if status_code in (HTTPForbidden.code, ):
            raise SchedulerClientHttpException(res, status_code)

        return res, status_code

    def update_timetable(self, timetable_id, data):
        """
        Updates timetable with periodic task.
        """
        url = urlparse.urljoin(self.BASE_URL, str(timetable_id))

        partial_update = (
            ('interval' not in data) and ('crontab' not in data))
        if partial_update:
            # updates only timetable-record.
            logger.debug(
                u'Sending PATCH /{0}{1}/ data:{2}'.format(self.BASENAME,
                                                          timetable_id, data))
            r = requests.patch(url, data=data, headres=self.headers)
            logger.info(u'PATCH /{0}{1}/ successed with {2}:{3}'.format(
                self.BASENAME, timetable_id, r.status_code, r.content))
        else:
            # update timetable-record removing previous periodic task and
            # adding new periodic task with giving params
            logger.debug(
                u'Sending PUT /{0}{1}/ data:{2}'.format(self.BASENAME,
                                                        timetable_id, data))
            r = requests.put(url, data=data, headers=self.headers)
            logger.info(u'PUT /{0}{1}/ successed with {2}:{3}'.format(
                self.BASENAME, timetable_id, r.status_code, r.content))

        return json.loads(r.content)

    def delete_timetable(self, timetable_id):
        """
        Delete timetable with periodic task.
        """
        r = requests.delete(
            urlparse.urljoin(self.BASE_URL, str(timetable_id)),
            headers=self.headers)
        logger.info(u'DELETE /{0}{1}/ successed with {2}:{3}'.format(
            self.BASENAME, timetable_id, r.status_code, r.content))
        return json.loads(r.content)

    @property
    def info(self):
        """
        Get module info
        :return:
        """
        r = requests.get(self.MODULE_BASE_URL, headers=self.headers)
        return json.loads(r.content).get('info', {})

    @property
    def version(self):
        """
        Get version of scheduler
        :return:
        """
        return self.info.get('version')
