# coding: utf-8

import os

from collections import defaultdict

import logging

from logging import _levelNames as levelNames

import yaml

logger = logging.getLogger(__name__)

duplicate_keys = set()
unused_variables = ['CINDER_VERSION', 'DB', 'RABBIT_QUEUES', 'KEYSTONE',
                    'NOVA_VERSION', 'ceilometer_service_name']
pass_keys = ['auth_password', 'PASSWORD', 'password']
protocols = ['http']


def duplicates_detect_constructor(loader, node, deep=False):
    """Detect duplicate keys.
       Function have no effect when using yaml.safe_load()
       Doesn't work correctly when the key names in different
       sections the same
    """
    entry = {}
    for key_node, value_node in node.value:
        key = loader.construct_object(key_node, deep=deep)
        value = loader.construct_object(value_node, deep=deep)

        if key in entry:
            duplicate_keys.add(key)

        entry[key] = value
    return loader.construct_mapping(node, deep)

yaml.add_constructor(yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
                     duplicates_detect_constructor)


def read_settings(settings):
    duplicate_keys.clear()

    with open(settings) as fh:
        base_config = yaml.load(fh)

        if duplicate_keys:
            logger.error(
                u'Duplicate options found in file {0}: {1}. '.format(
                    settings, list(duplicate_keys)))

    duplicate_keys.clear()

    return base_config


def merge_dicts(base_dict, update_dict):
    for key, value in update_dict.iteritems():
        base = base_dict.get(key)
        if isinstance(base, dict) and isinstance(value, dict):
            base_dict[key] = merge_dicts(base, value)
        else:
            base_dict[key] = value

    return base_dict


def update_module_settings(result_settings, gl, module_config_path,
                           base_config_path):
    path = defaultdict(dict)

    if os.path.exists(base_config_path):
        path = read_settings(base_config_path)
    if os.path.exists(module_config_path):
        path = merge_dicts(path, read_settings(module_config_path))

    for key, value in path.iteritems():
        update_key(result_settings, key, value, gl, log_changes=False)


def update_global_from_settings(module_settings, gl):
    """ Update globals from module settings """
    for key, value in module_settings.iteritems():
        update_key(gl, key, value, log_changes=True)


def update_key(config, key, value, gl=None, parents='', log_changes=False):
    """ Recursively update values of nested dict """
    base_config = gl or config
    if key in base_config:
        if isinstance(value, dict):
            for k, v in value.iteritems():
                glob = base_config[key] if gl else None
                formatted_key = '{0}{1}.'.format(parents, key)
                update_key(config[key], k, v, gl=glob,
                           parents=formatted_key, log_changes=log_changes)
        else:
            config[key] = get_valid_value(
                base_config, key, value, parents, log_changes=log_changes)
    elif key not in unused_variables:
        logger.error(u'{0}{1} is not found in settings.'.format(
            parents, key))


def get_valid_value(gl, key, value, parent='', log_changes=False):
    default = gl[key]
    if duplicate_keys and key in duplicate_keys:
        logger.error(
            u'Value for duplicate option "{0}{1}" '
            u'set to default: {2} '.format(parent, key, default))
        value = default
    else:
        if key in validators:
            value = validators[key](key, value, default)
        if default != value and log_changes:
            showed_default = default
            showed_value = value
            if key in pass_keys:
                showed_default = '*' * len(default)
                showed_value = '*' * len(value)
            logger.debug(u'Changed value of "{0}{1}" from "{2}"'
                         u' to "{3}"'.format(parent, key,
                                             showed_default,
                                             showed_value))
    return value


def get_valid_port(key, value=None, default_port=None):
    value = unicode(value)
    if not value.isdigit():
        logger.error(
            u'Invalid port: {0}. '
            u'Port must consist only of numbers. '
            u'Port set to default: {1}'.format(value, default_port))

        value = default_port
    return value


def get_valid_protocol(key, value, default_protocol):
    if not value:
        return default_protocol

    if value not in protocols:
        logger.error(
            u'Invalid protocol for scheduler url: {0}. '
            u'Protocol set to default: {1}'.format(
                value, default_protocol))

        value = default_protocol
    return value


def get_valid_log_level(key, value, default_log_level):
    processed_value = value.upper()
    available_log_levels = [_key for _key in levelNames
                            if isinstance(_key, basestring)]

    if processed_value not in available_log_levels:
        processed_value = default_log_level

        logger.error(
            u'Invalid log level: {0}. '
            u'Log level must be one of {1}. '
            u'Log level set to default: {2}'.format(
                value, available_log_levels, default_log_level))

    return processed_value


def get_valid_host(key, value=None, default_host=None):
    if value:
        value = unicode(value)
    else:
        logger.error(
            u'Invalid host: {0}. '
            u'Host set to default: {1}'.format(value, default_host))

        value = default_host
    return value


def get_valid_str(key, value, default):
    return unicode(value)


validators = {
    'LOG_LEVEL': get_valid_log_level,
    'PORT': get_valid_port,
    'HOST': get_valid_host,
    'PROTOCOL': get_valid_protocol,
    'auth_password': get_valid_str,
    'password': get_valid_str,
    'PASSWORD': get_valid_str,
}
